package cmd

import (
	"context"
	"flag"
	"fmt"
	"strings"

	"github.com/AlecAivazis/survey/v2"
	"github.com/go-git/go-git/v5"
	"github.com/go-git/go-git/v5/plumbing"
	"github.com/go-git/go-git/v5/plumbing/object"
	"github.com/peterbourgon/ff/v3/ffcli"
	"github.com/rs/zerolog/log"
)

func (h *Handler) Frontport() *ffcli.Command {
	fs := flag.NewFlagSet("frontport", flag.ContinueOnError)
	fromFlag := fs.String("from", "", "Release to frontport from (ex: `1.17`, default: <latest>)")
	fs.StringVar(fromFlag, "f", *fromFlag, "--from")
	toFlag := fs.String("to", "", "Release to frontport to (ex: `main`, default: `main`)")
	fs.StringVar(toFlag, "t", *toFlag, "--to")
	return &ffcli.Command{
		Name:       "frontport",
		FlagSet:    fs,
		ShortUsage: "frontport --from [release=latest] --to [release=main]",
		ShortHelp:  "frontport cherry-picks a commit and applies it to a clean branch based on `release`",
		Exec: func(ctx context.Context, _ []string) error {
			if err := h.checkInit(); err != nil {
				return err
			}

			from := *fromFlag
			if from == "" {
				from = h.latestRelease()
			}
			if !strings.HasPrefix(from, "release") {
				from = fmt.Sprintf("release/v%s", from)
			}

			h.fetch(ctx)

			if !isClean() {
				log.Fatal().Msg("working tree is dirty")
			}

			commits, err := h.repo().Log(&git.LogOptions{
				From: h.head(from),
			})
			if err != nil {
				return err
			}

			optMap := make(map[string]plumbing.Hash)
			var opts []string
			if err := commits.ForEach(func(c *object.Commit) error {
				title := strings.Split(c.Message, "\n")[0]
				opts = append(opts, title)
				optMap[title] = c.Hash
				return nil
			}); err != nil {
				return err
			}

			var resp string
			if err := survey.AskOne(&survey.Select{
				Message: "Commit to cherry-pick",
				Options: opts,
			}, &resp, survey.WithValidator(survey.Required)); err != nil {
				return err
			}

			hash := optMap[resp]
			branch := fmt.Sprintf("frontport-%s", hash)

			base := *toFlag
			if base == "" {
				base = "upstream/main"
			}
			if !strings.HasPrefix(base, "upstream") {
				base = fmt.Sprintf("upstream/release/v1.%s", base)
			}
			if err := h.Branch().ParseAndRun(ctx, []string{branch, base}); err != nil {
				return err
			}

			return run(ctx, h.Config.WorkspaceBranch(branch), "git", "cherry-pick", hash.String())
		},
	}
}
